@@ -1,12 +1,12 @@
 package Perinci::CmdLine::Classic;
 
-our $DATE = '2017-07-12'; # DATE
-our $VERSION = '1.75_001'; # VERSION
+our $DATE = '2015-02-06'; # DATE
+our $VERSION = '1.45'; # VERSION
 
 use 5.010001;
 #use strict; # enabled by Moo
 #use warnings; # enabled by Moo
-use Log::ger;
+use Log::Any '$log';
 
 use Moo;
 use experimental 'smartmatch'; # must be after Moo
@@ -22,6 +22,7 @@ with 'Color::Theme::Role::ANSI' unless $ENV{COMP_LINE};
 with 'Perinci::CmdLine::Classic::Role::Help' unless $ENV{COMP_LINE};
 with 'Term::App::Role::Attrs' unless $ENV{COMP_LINE};
 
+has log => (is => 'rw', default=>sub{1});
 has undo => (is=>'rw', default=>sub{0});
 has undo_dir => (
     is => 'rw',
@@ -78,11 +79,13 @@ has action_metadata => (
             clear_history => {
             },
             help => {
+                default_log => 0,
                 use_utf8 => 1,
             },
             history => {
             },
             subcommands => {
+                default_log => 0,
                 use_utf8 => 1,
             },
             redo => {
@@ -92,6 +95,7 @@ has action_metadata => (
             undo => {
             },
             version => {
+                default_log => 0,
                 use_utf8 => 1,
             },
         },
@@ -99,6 +103,9 @@ has action_metadata => (
 );
 has default_prompt_template => (is=>'rw');
 
+# OLD name for backward compat, will be removed later
+sub log_any_app { goto \&log }
+
 sub VERSION {
     my ($pkg, $req) = @_;
     $REQ_VERSION = $req;
@@ -119,12 +126,15 @@ sub BUILD {
     if (!$self->{actions}) {
         $self->{actions} = {
             version => {
+                default_log => 0,
                 use_utf8 => 1,
             },
             help => {
+                default_log => 0,
                 use_utf8 => 1,
             },
             subcommands => {
+                default_log => 0,
                 use_utf8 => 1,
             },
             call => {},
@@ -163,31 +173,27 @@ sub BUILD {
             $_t->('help'),
             show_in_options => sub { $ENV{VERBOSE} },
         };
-
-        unless ($self->skip_format) {
-            $copts->{format} = {
-                $_t->('format'),
-                schema => ['str*' => in => $formats],
-            };
-            $copts->{json} = {
-                $_t->('json'),
-                summary => N__("Equivalent to --format=json-pretty"),
-            };
-            $copts->{naked_res} = {
-                $_t->('naked_res'),
-                summary => N__("When outputing as JSON, strip result envelope"),
-            };
-            $copts->{format_options} = {
-                getopt  => "format-options=s",
-                summary => N__("Pass options to formatter"),
-                handler => sub {
-                    my ($go, $val, $r) = @_;
-                    $r->{format_options} = __json_decode($val);
-                },
-                is_settable_via_config => 1,
-                tags => ['category:output'],
-            };
-        }
+        $copts->{format} = {
+            $_t->('format'),
+            schema => ['str*' => in => $formats],
+        };
+        # XXX support --naked-res
+        $copts->{json} = {
+            $_t->('json'),
+            summary => N__("Equivalent to --format=json-pretty"),
+            handler => sub {
+                my ($go, $val, $r) = @_;
+                $r->{format} = 'json-pretty';
+            },
+        };
+        $copts->{format_options} = {
+            getopt  => "format-options=s",
+            summary => N__("Pass options to formatter"),
+            handler => sub {
+                my ($go, $val, $r) = @_;
+                $r->{format_options} = __json_decode($val);
+            },
+        };
 
         if ($self->subcommands) {
             $copts->{subcommands} = {
@@ -210,10 +216,6 @@ sub BUILD {
             $copts->{config_profile} = { $_t->('config_profile') };
         }
 
-        if ($self->read_env) {
-            $copts->{no_env} = { $_t->('no_env') };
-        }
-
         if ($self->log) {
             $copts->{log_level} = { $_t->('log_level'), };
             $copts->{trace}     = { $_t->('trace'), };
@@ -224,6 +226,7 @@ sub BUILD {
 
         if ($self->undo) {
             $copts->{history} = {
+                category => 'Undo options',
                 getopt  => 'history',
                 summary => N__('List actions history'),
                 handler => sub {
@@ -231,9 +234,9 @@ sub BUILD {
                     $r->{action} = 'history';
                     $r->{skip_parse_subcommand_argv} = 1;
                 },
-                tags => ['category:undo'],
             };
             $copts->{clear_history} = {
+                category => 'Undo options',
                 getopt  => "clear-history",
                 summary => N__('Clear actions history'),
                 handler => sub {
@@ -241,9 +244,9 @@ sub BUILD {
                     $r->{action} = 'clear_history';
                     $r->{skip_parse_subcommand_argv} = 1;
                 },
-                tags => ['category:undo'],
             };
             $copts->{undo} = {
+                category => 'Undo options',
                 getopt  => 'undo',
                 summary => N__('Undo previous action'),
                 handler => sub {
@@ -251,9 +254,9 @@ sub BUILD {
                     $r->{action} = 'undo';
                     $r->{skip_parse_subcommand_argv} = 1;
                 },
-                tags => ['category:undo'],
             };
             $copts->{redo} = {
+                category => 'Undo options',
                 getopt  => 'redo',
                 summary => N__('Redo previous undone action'),
                 handler => sub {
@@ -286,14 +289,14 @@ sub BUILD {
 }
 
 sub __json_decode {
-    require JSON::MaybeXS;
-    state $json = do { JSON::MaybeXS->new->allow_nonref };
+    require JSON;
+    state $json = do { JSON->new->allow_nonref };
     $json->decode(shift);
 }
 
 sub __json_encode {
-    require JSON::MaybeXS;
-    state $json = do { JSON::MaybeXS->new->allow_nonref };
+    require JSON;
+    state $json = do { JSON->new->allow_nonref };
     $json->encode(shift);
 }
 
@@ -337,26 +340,14 @@ sub hook_format_row {
 sub hook_after_get_meta {
     my ($self, $r) = @_;
 
-    my $metao = risub($r->{meta});
-    if ($metao->can_dry_run) {
-        my $default_dry_run = $metao->default_dry_run // $self->default_dry_run;
-        $r->{dry_run} = 1 if $default_dry_run;
-        $r->{dry_run} = ($ENV{DRY_RUN} ? 1:0) if defined $ENV{DRY_RUN};
+    if (risub($r->{meta})->can_dry_run) {
         $self->common_opts->{dry_run} = {
-            getopt  => $default_dry_run ? 'dry-run!' : 'dry-run',
-            summary => $default_dry_run ?
-                N__("Disable simulation mode (also via DRY_RUN=0)") :
-                N__("Run in simulation mode (also via DRY_RUN=1)"),
+            getopt  => 'dry-run',
+            summary => N__("Run in simulation mode (also via DRY_RUN=1)"),
             handler => sub {
                 my ($go, $val, $r) = @_;
-                if ($val) {
-                    log_debug("[pericmd] Dry-run mode is activated");
-                    $r->{dry_run} = 1;
-                    #$ENV{VERBOSE} = 1;
-                } else {
-                    log_debug("[pericmd] Dry-run mode is deactivated");
-                    $r->{dry_run} = 0;
-                }
+                $r->{dry_run} = 1;
+                $ENV{VERBOSE} = 1;
             },
         };
     }
@@ -443,17 +434,47 @@ sub _unsetup_progress_output {
     $setup_progress = 0;
 }
 
+sub _load_log_any_app {
+    my ($self, $r) = @_;
+    # Log::Any::App::init can already avoid being run twice, but we need to
+    # check anyway to avoid logging starting message below twice.
+    return if $r->{_log_any_app_loaded}++;
+    require Log::Any::App;
+    Log::Any::App::init();
+
+    # we log this after we initialize Log::Any::App, since Log::Any::App might
+    # not be loaded at all. yes, this means that this log message is printed
+    # rather late and might not be the first message to be logged (see log
+    # messages in run()) if user already loads Log::Any::App by herself.
+    $log->debugf("Program %s started with arguments: %s",
+                 $0, $r->{orig_argv});
+}
+
 # this hook is called at the start of run(), can be used to initialize stuffs
 sub hook_before_run {
     my ($self, $r) = @_;
 
-    log_trace("Start of CLI run");
+    $log->tracef("Start of CLI run");
 
     # save, for showing in history, among others
     $r->{orig_argv} = [@ARGV];
 }
 
 sub hook_after_parse_argv {
+    my ($self, $r) = @_;
+
+    # We load Log::Any::App rather late here, so user can customize level via
+    # --debug, --dry-run, etc.
+    unless ($ENV{COMP_LINE}) {
+        my $do_log = $r->{subcommand_data}{log} //
+            $r->{subcommand_data}{log_any_app} # OLD compat, will be removed later
+                if $r->{subcommand_data};
+        $do_log //= $ENV{LOG};
+        $do_log //= $self->action_metadata->{$r->{action}}{default_log}
+            if $self->{action};
+        $do_log //= $self->log;
+        $self->_load_log_any_app($r) if $do_log;
+    }
 }
 
 sub hook_format_result {
@@ -473,60 +494,55 @@ sub hook_format_result {
         $format = 'text';
     }
 
-    $res->[3]{format_options} = $r->{format_options} if $r->{format_options};
+    $res->[3]{result_format_options} = $r->{format_options}
+        if $r->{format_options};
 
-    my $fres;
     if ($res->[3]{is_stream}) {
-        log_trace("Result is a stream");
+        $log->tracef("Result is a stream");
         return undef;
     } elsif ($res->[3]{'x.hint.result_binary'} && $format =~ /text/) {
-        $fres = $res->[2];
+        $r->{fres} = $res->[2];
     } else {
-        log_trace("Formatting output with %s", $format);
-        $fres = Perinci::Result::Format::format($res, $format, $r->{naked_res});
+        $log->tracef("Formatting output with %s", $format);
+        $r->{fres} = Perinci::Result::Format::format($res, $format);
     }
-
-    # ux: prefix error message with program name
-    if ($format=~/text/ && $r->{res}[0] =~ /\A[45]/ && defined($r->{res}[1])) {
-        $fres = "$self->{program_name}: $fres";
-    }
-
-    $fres;
 }
 
 sub hook_display_result {
     my ($self, $r) = @_;
 
     my $res  = $r->{res};
+    my $fres = $r->{fres};
     my $resmeta = $res->[3] // {};
 
     my $handle = $r->{output_handle};
 
+    # determine whether to binmode(STDOUT,":utf8")
+    my $utf8;
+    {
+        last if defined($utf8 = $ENV{UTF8});
+        if ($resmeta->{'x.hint.result_binary'}) {
+            # XXX only when format is text?
+            $utf8 = 0; last;
+        }
+        my $am = $self->action_metadata->{$r->{action}}
+            if $r->{action};
+        last if defined($utf8 //= $am->{use_utf8});
+
+        if ($r->{subcommand_data}) {
+            last if defined($utf8 //= $r->{subcommand_data}{use_utf8});
+        }
+
+        $utf8 //= $self->use_utf8
+            if $self->can("use_utf8");
+    }
+    binmode(STDOUT, ":utf8") if $utf8;
+
     if ($ENV{COMP_LINE} || $res->[3]{"cmdline.skip_format"}) {
         print $handle $res->[2];
         return;
     }
 
-    # set utf8 flag
-    my $utf8;
-    {
-        if ($resmeta->{'x.hint.result_binary'}) {
-            # XXX only when format is text?
-            $utf8 = 0; last;
-        }
-
-        my $am = $self->action_metadata->{$r->{action}}
-            if $r->{action};
-        last if defined($utf8 = $am->{use_utf8});
-
-        if ($r->{subcommand_data}) {
-            last if defined($utf8 = $r->{subcommand_data}{use_utf8});
-        }
-
-        $utf8 = $self->use_utf8;
-    }
-    binmode($handle, ":utf8") if $utf8;
-
     $self->display_result($r);
 }
 
@@ -535,12 +551,12 @@ sub hook_after_run {
     $self->_unsetup_progress_output;
 }
 
-sub action_subcommands {
+sub run_subcommands {
     my ($self, $r) = @_;
 
     if (!$self->subcommands) {
         return [200, "OK", __("There are no subcommands") . ".",
-                {"cmdline.skip_format"=>1}];
+                {"x.perinci.cmdline._skip_format"=>1}];
     }
 
     $r->{_help_buf} = '';
@@ -585,63 +601,37 @@ sub action_subcommands {
     $self->_help_draw_curtbl($r);
 
     [200, "OK", $r->{_help_buf},
-     {"cmdline.skip_format"=>1}];
+     {"x.perinci.cmdline._skip_format"=>1}];
 }
 
-sub action_version {
-    no strict 'refs';
-
+sub run_version {
     my ($self, $r) = @_;
 
     my $url = $r->{subcommand_data}{url} // $self->url;
+    my $meta = $self->get_meta($r, $url);
+    my $ver = $meta->{entity_v} // "?";
+    my $date = $meta->{entity_date};
 
-    my @text;
-
-    {
-        my $meta = $self->get_meta($r, $url);
-        push @text, __x(
+    [200, "OK", join(
+        "",
+        __x(
             "{program} version {version}",
             program => $self->_color('program_name',
                                      $self->get_program_and_subcommand_name),
-            version => $self->_color('emphasis', ($meta->{entity_v} // "?")),
-        ),
-            ($meta->{entity_date} ? " ($meta->{entity_date})" : ""),
-            "\n";
-        for my $mod (@{ $meta->{'x.dynamic_generator_modules'} // [] }) {
-            push @text, "  ", __x(
-                "{program} version {version}",
-                program => $self->_color('emphasis', $mod),
-                version => $self->_color('emphasis', (${"$mod\::VERSION"} // "?")),
-            ),
-                (${"$mod\::DATE"} ? " (".${"$mod\::DATE"}.")" : ""),
-                    "\n";
-        }
-    }
-
-    for my $url (@{ $self->extra_urls_for_version // [] }) {
-        my $meta = $self->get_meta($r, $url);
-        push @text, "  ", __x(
+            version => $self->_color('emphasis', $ver)),
+        ($date ? " ($date)" : ""),
+        "\n",
+        "  ", __x(
             "{program} version {version}",
-            program => $self->_color('emphasis', $url),
-            version => $self->_color('emphasis', ($meta->{entity_v} // "?")),
-        ),
-            ($meta->{entity_date} ? " ($meta->{entity_date})" : ''),
-            "\n";
-    }
-
-    push @text, "  ", __x(
-        "{program} version {version}",
-        program => $self->_color('emphasis', "Perinci::CmdLine::Classic"),
-        version => $self->_color('emphasis',
-                                 $Perinci::CmdLine::Classic::VERSION || "dev"),
-    ),
+            program => $self->_color('emphasis', "Perinci::CmdLine::Classic"),
+            version => $self->_color('emphasis',
+                                     $Perinci::CmdLine::Classic::VERSION || "dev")),
         ($Perinci::CmdLine::Classic::DATE ? " ($Perinci::CmdLine::Classic::DATE)" : ""),
-        "\n";
-
-    [200, "OK", join("", @text), {"cmdline.skip_format"=>1}];
+        "\n",
+    ), {"x.perinci.cmdline._skip_format"=>1}];
 }
 
-sub action_call {
+sub run_call {
     my ($self, $r) = @_;
 
     my $scn = $r->{subcommand_name};
@@ -690,7 +680,7 @@ sub action_call {
             call => $scd->{url},
             {args=>\%fargs, tx_id=>$tx_id, dry_run=>$dry_run});
     }
-    log_trace("call res=%s", $res);
+    $log->tracef("call res=%s", $res);
 
     # commit transaction (if using tx)
     if ($using_tx && $res->[0] =~ /\A(?:200|304)\z/) {
@@ -703,10 +693,10 @@ sub action_call {
     $res;
 }
 
-sub action_history {
+sub run_history {
     my ($self, $r) = @_;
     my $res = $self->riap_client->request(list_txs => "/", {detail=>1});
-    log_trace("list_txs res=%s", $res);
+    $log->tracef("list_txs res=%s", $res);
     return $res unless $res->[0] == 200;
     $res->[2] = [sort {($b->{tx_commit_time}//0) <=> ($a->{tx_commit_time}//0)}
                      @{$res->[2]}];
@@ -725,17 +715,17 @@ sub action_history {
     [200, "OK", \@txs];
 }
 
-sub action_clear_history {
+sub run_clear_history {
     my ($self, $r) = @_;
     $self->riap_client->request(discard_all_txs => "/");
 }
 
-sub action_undo {
+sub run_undo {
     my ($self, $r) = @_;
     $self->riap_client->request(undo => "/");
 }
 
-sub action_redo {
+sub run_redo {
     my ($self, $r) = @_;
     $self->riap_client->request(redo => "/");
 }
@@ -755,132 +745,15 @@ Perinci::CmdLine::Classic - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-This document describes version 1.75_001 of Perinci::CmdLine::Classic (from Perl distribution Perinci-CmdLine-Classic), released on 2017-07-12.
+This document describes version 1.45 of Perinci::CmdLine::Classic (from Perl distribution Perinci-CmdLine-Classic), released on 2015-02-06.
 
 =head1 SYNOPSIS
 
-In C<gen-random-num> script:
-
- use Perinci::CmdLine::Classic;
-
- our %SPEC;
-
- $SPEC{gen_random_num} = {
-     v => 1.1,
-     summary => 'Generate some random numbers',
-     args => {
-         count => {
-             summary => 'How many numbers to generate',
-             schema => ['int*' => min=>0],
-             default => 1,
-             cmdline_aliases=>{n=>{}},
-             req => 1,
-             pos => 0,
-         },
-         min => {
-             summary => 'Lower limit of random number',
-             schema => 'float*',
-             default => 0,
-         },
-         max => {
-             summary => 'Upper limit of random number',
-             schema => 'float*',
-             default => 1,
-         },
-     },
-     result_naked => 1,
- };
- sub gen_random_num {
-     my %args = @_;
-
-     my @res;
-     for (1..$args{count}) {
-         push @res, $args{min} + rand()*($args{max}-$args{min});
-     }
-     \@res;
- }
-
- Perinci::CmdLine::Classic->new(url => '/main/gen_random_num')->run;
-
-Run your script:
-
- % ./gen-random-num
- 0.999473691060306
-
- % ./gen-random-num --min 1 --max 10 5
- 1.27390166158969
- 1.69077475473679
- 8.97748327778684
- 5.86943773494068
- 8.34341298182493
-
-JSON output support out of the box:
-
- % ./gen-random-num -n3 --json
- [200,"OK (envelope added by Perinci::Access::Lite)",[0.257073684902029,0.393782991540746,0.848740540017513],{}]
-
-Automatic help message:
-
- % ./gen-random-num -h
- gen-random-num - Generate some random numbers
-
- Usage:
-   gen-random-num --help (or -h, -?)
-   gen-random-num --version (or -v)
-   gen-random-num [options] [count]
- Options:
-   --config-path=s     Set path to configuration file
-   --config-profile=s  Set configuration profile to use
-   --count=i, -n       How many numbers to generate (=arg[0]) [1]
-   --format=s          Choose output format, e.g. json, text [undef]
-   --help, -h, -?      Display this help message
-   --json              Set output format to json
-   --max=f             Upper limit of random number [1]
-   --min=f             Lower limit of random number [0]
-   --naked-res         When outputing as JSON, strip result envelope [0]
-   --no-config         Do not use any configuration file
-   --version, -v
-
-Automatic configuration file support:
-
- % cat ~/gen-random-num.conf
- count=5
- max=0.01
-
- % ./gen-random-num
- 0.00105268954838724
- 0.00701443611501844
- 0.0021247476506154
- 0.00813872824513005
- 0.00752832346491306
-
-Automatic tab completion support:
-
- % complete -C gen-random-num gen-random-num
- % gen-random-num --mi<tab>
-
-See L<Perinci::CmdLine::Manual> for details on other available features
-(subcommands, automatic formatting of data structures, automatic schema
-validation, dry-run mode, automatic POD generation, remote function support,
-automatic CLI generation, automatic --version, automatic HTTP API,
-undo/transactions, configurable output format, logging, progress bar,
-colors/Unicode, and more).
+See L<Perinci::CmdLine::Manual::Examples>.
 
 =head1 DESCRIPTION
 
-Perinci::CmdLine is a command-line application framework. It allows you to
-create full-featured CLI applications easily and quickly.
-
-See L<Perinci::CmdLine::Manual> for more details.
-
-There is also a blog post series on Perinci::CmdLine tutorial:
-L<https://perlancar.wordpress.com/category/pericmd-tut/>
-
-Perinci::CmdLine::Classic is the heavier backend implementation which supports
-some extra features currently not supported by the default backend
-implementation L<Perinci::CmdLine::Lite>. These features come at some startup
-overhead cost and more dependencies. You normally should use
-L<Perinci::CmdLine::Any> instead to be able to switch backend on the fly.
+See L<Perinci::CmdLine::Manual>.
 
 Screenshots:
 
@@ -913,19 +786,15 @@ hash/stash.
 
 =back
 
-=head1 RESULT METADATA
-
-All those supported by L<Perinci::CmdLine::Base>, plus:
-
-=head2 x.hint.result_binary => bool
-
-If set to true, then when formatting to C<text> formats, this class won't print
-any newline to keep the data being printed unmodified.
-
 =head1 ATTRIBUTES
 
 All the attributes of L<Perinci::CmdLine::Base>, plus:
 
+=head2 log => BOOL (default: 1)
+
+Whether to load L<Log::Any::App> (enable logging output) by default. See
+L</"LOGGING"> for more details.
+
 =head2 use_utf8 => BOOL
 
 From L<Term::App::Role::Attrs> (please see its docs for more details). There are
@@ -970,6 +839,15 @@ All the methods of L<Perinci::CmdLine::Base>, plus:
 
 =back
 
+=head1 RESULT METADATA
+
+All those supported by L<Perinci::CmdLine::Base>, plus:
+
+=head2 x.hint.result_binary => bool
+
+If set to true, then when formatting to C<text> formats, this class won't print
+any newline to keep the data being printed unmodified.
+
 =head1 ENVIRONMENT
 
 All the environment variables that L<Perinci::CmdLine::Base> supports, plus:
@@ -990,6 +868,17 @@ Please see L<Term::App::Role::Attrs>.
 
 Please see L<Term::App::Role::Attrs>.
 
+=head1 SEE ALSO
+
+L<Perinci>, L<Rinci>, L<Riap>.
+
+L<Perinci::CmdLine::Base>.
+
+L<Perinci::CmdLine::Lite>.
+
+Other CPAN modules to write command-line applications: L<App::Cmd>, L<App::Rad>,
+L<MooseX::Getopt>.
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Perinci-CmdLine-Classic>.
@@ -1006,17 +895,13 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
-=head1 SEE ALSO
-
-L<Perinci::CmdLine::Any>, L<Perinci::CmdLine::Lite>.
-
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017, 2016, 2015, 2014, 2013, 2012, 2011 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
