@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Classic::Role::Help;
 
-our $DATE = '2017-07-12'; # DATE
-our $VERSION = '1.75_001'; # VERSION
+our $DATE = '2015-02-06'; # DATE
+our $VERSION = '1.45'; # VERSION
 
 # split here just so it's more organized
 
@@ -68,18 +68,14 @@ sub _help_add_row {
 
     $t->add_row($row);
 
-    my $dux_available = eval { require Data::Unixish; 1 } && !$@;
-
-    if ($dux_available) {
-        for (0..@{$t->{columns}}-1) {
-            my %styles = (formats=>[]);
-            push @{ $styles{formats} },
-                [wrap=>{ansi=>1, mb=>1, width=>$t->{cell_width}-$indent*2}]
+    for (0..@{$t->{columns}}-1) {
+        my %styles = (formats=>[]);
+        push @{ $styles{formats} },
+            [wrap=>{ansi=>1, mb=>1, width=>$t->{cell_width}-$indent*2}]
                 if $wrap;
-            push @{ $styles{formats} }, [lins=>{text=>"  " x $indent}]
-                if $indent && $_ == 0;
-            $t->set_cell_style($rownum, $_, \%styles);
-        }
+        push @{ $styles{formats} }, [lins=>{text=>"  " x $indent}]
+            if $indent && $_ == 0;
+        $t->set_cell_style($rownum, $_, \%styles);
     }
 }
 
@@ -262,8 +258,8 @@ sub help_section_subcommands {
     my ($self, $r) = @_;
 
     my $verbose = $r->{_help_verbose};
-    return unless $self->subcommands && !$r->{subcommand_name};
-    my $scs = $self->list_subcommands;
+    my $scs = $self->subcommands;
+    return unless $scs && !$r->{subcommand_name};
 
     my @scs = sort keys %$scs;
     my @shown_scs;
@@ -423,7 +419,7 @@ sub help_section_links {
     # not yet
 }
 
-sub action_help {
+sub run_help {
     my ($self, $r) = @_;
 
     $r->{_help_buf} = '';
@@ -497,7 +493,7 @@ sub action_help {
         $self->$meth($r);
     }
     $self->_help_draw_curtbl($r);
-    [200, "OK", $r->{_help_buf}, {"cmdline.skip_format"=>1}];
+    [200, "OK", $r->{_help_buf}, {"x.perinci.cmdline.skip_format"=>1}];
 }
 
 1;
@@ -515,7 +511,7 @@ Perinci::CmdLine::Classic::Role::Help - Help-related routines
 
 =head1 VERSION
 
-This document describes version 1.75_001 of Perinci::CmdLine::Classic::Role::Help (from Perl distribution Perinci-CmdLine-Classic), released on 2017-07-12.
+This document describes version 1.45 of Perinci::CmdLine::Classic::Role::Help (from Perl distribution Perinci-CmdLine-Classic), released on 2015-02-06.
 
 =for Pod::Coverage ^(.+)$
 
@@ -552,7 +548,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017, 2016, 2015, 2014, 2013, 2012, 2011 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
